{=========================================================================}
{V24M Multiport-UNIT zur interruptgesteuerten Empfangsabfrage von         }
{seriellen IBM-PC V24-Schnittstellen.                                     }
{                                                                         }
{Es koennen 4 COM-Schnittstellen und ein TFPCX-Interface bedient werden.  }
{Die Hardwaredaten fuer sind fuer jede COM-Schnittstelle individuell      }
{einstellbar. Es werden beliebige Basisadressen und die IRQs von 1-15     }
{unterstuetzt.                                                            }
{                                                                         }
{Ist ein UART vom Typ NSC16550A installiert, so wird dessen FIFO Speicher }
{eingeschaltet um Overrun-Fehler unter unguenstigen Systemkonfigurationen }
{zu vermeiden.                                                            }
{                                                                         }
{Bei Eingabe der Schnittstelle COM5 fuer einen Port wird das TFPCR-Soft-  }
{wareinterface von OM Sigi Kluger DL1MEN in der Implementierung von OM    }
{Ren Stange DG0FT unterstuetzt.                                          }
{Damit koennen Programme welche die UNIT V24 verwenden ohne grosse Aende- }
{rungen mit dem TFPCX und externem MODEM ohne TNC benutzt werden.         }
{                                                                         }
{                 V7.7 12.04.93 DL5FBD Gerd Michael Ritter                }
{=========================================================================}

UNIT V24M;

Interface

USES CRT,  {Fuer Pascal Systemzeitgeberfunktion Delay            }
     DOS;  {Fuer Zugriffe auf Interruptvektoren von BIOS und DOS }

{-------------------------------------------------------------------------}
{Version der UNIT V24M. Kann vom Anwenderprogramm angefragt werden.       }
{-------------------------------------------------------------------------}

Const     V24MVersion  ='7.7';                 {Version der UNIT}

{-------------------------------------------------------------------------}
{Prozeduren und Funktionen die dem Benutzer zugaenglich sind.             }
{Es gibt keine gemeinsammen Variablen. Das Setzen oder Lesen von Variablen}
{der V24 UNIT erfolgt ueber Interfaceprozeduren oder Interfacefunktionen. }
{Damit wird verhindert, das ein Benutzerprogramm an internen Variablen der}
{UNIT "herumpfuschen" kann, was sehr unangenehme Folgen haben kann.       }
{-------------------------------------------------------------------------}


Procedure V24DefHW    (SER         :Byte;      {ext. HW-Definition}
                       Basis       :Word;
                       IRQ         :Byte);
Procedure V24DefFCR   (SER         :Byte;      {FCR Config Bytes}
                       FCRData     :Byte);
Procedure V24OPEN     (SER         :Byte;      {Parameterdefinition}
                       BAUD        :Real;      {fuer Schnittstelle}
                       Bitzahl     :Byte;      {und Handshake}
                       Paritaet    :Char;
                       Stopbit     :Byte;
                       Handshake   :Char);
Procedure V24CLOSE    (SER         :Byte);     {Schnittstelle schlie.}
Procedure SendBreak   (SER         :Byte;      {Break senden}
                       Zeit          :Word);
Procedure V24Write    (SER         :Byte;      {Zeichen ausgeben}
                       Zeichen       :Char);
Procedure V24SetTXR   (SER         :Byte);     {SET TXReady=TRUE}
Procedure SetDTR      (SER         :Byte);     {DTR-Leitung auf log.1}
Procedure SetRTS      (SER         :Byte);     {RTS-Leitung auf log.1}
Procedure ClearDTR    (SER         :Byte);     {DTR-Leitung auf log.0}
Procedure ClearRTS    (SER         :Byte);     {RTS-Leitung auf log.0}

Function V24Install  (Test:Byte)   :Boolean;   {HW-Install.-Pruefung}
Function V24Count    (SER:Byte)    :Word;      {Zeichenzahl im Puffer}
Function V24Read     (SER:Byte)    :Char;      {Zeichen lesen}
Function V24Aktive   (SER:Byte)    :Boolean;   {UNIT-Aktiv-Status}
Function V24Overrun  (SER:Byte)    :Boolean;   {Ueberlauf Ringpuffer}
Function TXReady     (SER:Byte)    :Boolean;   {TX-Handshakestatus}
Function RXReady     (SER:Byte)    :Boolean;   {RX-Handshakestatus}
Function ReadDSR     (SER:Byte)    :Byte;      {Lesen DSR-Zustand}
Function ReadCTS     (SER:Byte)    :Byte;      {Lesen CTS-Rustand}
Function ReadDCD     (SER:Byte)    :Byte;      {Lesen DCD-Zustand}
Function ReadRI      (SER:Byte)    :Byte;      {Lesen RI-Zustand}


Implementation

{-------------------------------------------------------------------------}
{Definition der seriellem Schnittstellen mit Basisadresse und IRQ gemaess }
{einer sinnvollen Einstellung von COM3 und COM4 fuer Multiportbetrieb.    }
{                                                                         }
{       COM1 Basisadresse=3F8 IRQ=4     COM2 Basisadresse=2F8 IRQ=3       }
{       COM3 Basisadresse=3E8 IRQ=5     COM4 Basisadresse=2E8 IRQ=9       }
{                                                                         }
{Man beachte hierbei, das ein Anwenderprogramm mit der Procedure V24DefHW }
{die Definitionen fuer (COM3 und COM4) aendern kann.                      }
{-------------------------------------------------------------------------}

CONST   COMPort         :Array[1..4]  of WORD = ($3F8,$2F8,$3E8,$2E8);
        COMIRQ          :Array[1..4]  of BYTE = (04,03,05,09);

{Hardwaredaten,Uebertragungsparameter und Bitmaskierungskonstanten}

        {Interruptvektoren des PCs als Index des IRQ}
        COMInt          :Array[1..15] of BYTE = ($09,$0a,$0b,$0c,$0d,
                                                 $0e,$0f,$70,$71,$72,
                                                 $73,$74,$75,$76,$77);
        {Bitmaske fuer 8259 Maskenregister als Index des IRQ}
        IntMask         :Array[1..15] of BYTE = ($02,$04,$08,$10,$20,
                                                 $40,$80,$01,$02,$04,
                                                 $08,$10,$20,$40,$80);
        {Basisadressen der 8259 Interruptcontroller [1]=XT/AT [2]=nur AT}
        Port8259        :Array[1..2]  of BYTE = ($20,$A0);

        {16550-FIFO Control Register Konfigurationsbytes fuer COM1-COM4}
        FCRByte         :Array[1..4] of BYTE = ($81,$81,$81,$81);

        {Inititialisierungsstatus der Schnittstellenhandler fuer COM1-COM5}
        V24Init         :Array[1..5] of BOOLEAN =(false,false,false,
                                                  false,false);

        EOI8259         =$20;   {Non specific End of Interrupt fuer 8259}
        V24PrioMask     =$C1;   {IRQ-Prioritaet fuer RS232-Handler}
        STDPrioMask     =$C7;   {Standard System-IRQ-Prioritaet}
        ICRoff          =1;     {Interruptcontrolregister Interruptcontr.}
        DTRMask         =1;     {Maske zum Setzen des DTR-Bit}
        RTSMask         =2;     {Maske zum Setzen des RTS-Bit}
        DSRMask         =32;    {DSR-Bit-Lesemaske im MSR}
        CTSMask         =16;    {CTS-Bit-Lesemaske im MSR}
        DCDMask         =128;   {DCD-Bit-Lesemaske im MSR}
        RIMask          =64;    {RI -Bit-Lesemaske im MSR}
        RDRMask         =01;    {Receiver Data Readybit im LSR}
        LODIVoff        =00;    {Adressoffset Baudratenteiler niederw. Byte}
        HIDIVoff        =01;    {Adressoffset Baudratenteiler hoeherw. Byte}
        ENDIVMask       =128;   {Baudratenteilerzugriffsmaske Bit 2^7 LCR}
        IERoff          =1;     {IER hat gleiche Adr. wie HIBAUD}
        IIRoff          =02;    {Interrupt Identification Registeroffset}
        FCRoff          =02;    {FIFO Control Registeroffset beim 16550A}
        LCRoff          =03;    {Line Control Registeroffset}
        LSRoff          =05;    {Offset Line Status Register}
        MCRoff          =04;    {Modem Control Registeroffset}
        MSRoff          =06;    {Modem Status Registeroffset}
        IRQMask         =08;    {Modemcontrolregistermaske Bit 2^3}
        XON             =$11;   {XON bei Softwarehandshake}
        XOFF            =$13;   {XOFF bei Softwarehandshake}

        SIZE=4096;      {Schnittstellenpuffergroesse}
        Reserve=256;    {Ausloeseabstand zum Pufferende fuer Handshakesperre}

VAR     OrgIntMask      :Array[1..4,1..2] of BYTE; 
        OrgIntVek       :Array[1..4,1..2] of Pointer;
        PUFFER          :Array[1..5] of WORD;  {Anzahl Zeichen im Interruptpuffer}
        SER             :Byte;  {Gewaehlte COM-Schnittstelle}
        Hard            :Array[1..5] of BOOLEAN; {Indikator fuer RTS-CTS-Handshake}
        Soft            :Array[1..5] of BOOLEAN; {Indikator fuer XON-XOFF-Handshake}
        TXHand          :Array[1..5] of BOOLEAN; {Indikator ob Zeichensendung zulaessig ist}
                                  {oder Sendesperre durch Handshake vorliegt}
        RXHand          :Array[1..5] of BOOLEAN; {Indikator ob eigener Interrupthandler eine}
                                  {Handshakesperre aktiviert hatte}
        Overrun         :Array[1..5] of BOOLEAN; {Indikator fuer Ueberlauf des RX-Ringpuffer}
        OrgLODIV        :Array[1..4] of BYTE;  {Original Baudratenteiler LOW -Byte}
        OrgHIDIV        :Array[1..4] of BYTE;  {Original Baudratenteiler HIGH-Byte}
        OrgLCR          :Array[1..4] of BYTE;  {Original Line Control Register}
        OrgMCR          :Array[1..4] of BYTE;  {Original Modem Control Register}
        OrgIER          :Array[1..4] of BYTE;  {Original Interrupt Enable Register}
        BAUD            :Real;  {Baudrate}
        BitZahl         :Byte;  {Wortlaenge}
        Paritaet        :Char;  {Paritaet 'K'=keine 'U'=ungerade 'G'=gerade}
        Stopbit         :Byte;  {Stopbits 1=1Stopbit 2=2Stopbit}
        Handshake       :Char;  {H=Hardware S=Software K=Ringpuffer ohne HS}
        V24Puffer       :Array[1..5,1..SIZE] of BYTE;
        INTZEI          :Array[1..5] of WORD;  {Zeiger auf einzulesendes Zeichen}
        LESZEI          :Array[1..5] of WORD;  {Nachlaufender Auslesezeiger}
        IntCont         :Byte;  {Zuordnung IRQ<->8259 1=IRQ 1-7 2=IRQ 8-15}
        CPUREG          :Registers; {CPU-Registerrecord fuer INTR-Procedure}
        TFPCR_INT       :Byte;  {Kommunikationsinterrupt TFPCR-Interface}


{-------------------------------------------------------------------------}
{ TFPCR-Installations und Interrupterkennung                              }
{Interne Variablen:                                                       }
{I1,I2   :Feldindezes                                                     }
{Segm    :Segmentadresse fuer Speicherzugriff                             }
{Offs    :Offset fuer Speicherzugriff                                     }
{Test    :Vergleichsstring zur Erkennung                                  }
{Globale Variable:                                                        }
{TFPCR_INT :Kommunikationsinterrupt der TFPCx Treiber                     }
{-------------------------------------------------------------------------}

Function TFPCRInstall:Boolean;

VAR I1   :BYTE;
    I2   :Byte;
    Segm :Word;
    Offs :Word;
    TEST :String[4];

BEGIN
  TFPCR_INT:=0;
  FOR I1:=$40 TO $FF DO
    BEGIN
      Test:='';
      Segm:=MemW[0000:(I1*4+2)];
      Offs:=MemW[0000:I1*4];
      I2:=3;          
      WHILE (I2<7) AND (TFPCR_INT=0) DO
        BEGIN
          Test:=Test+Upcase(Chr(Mem[Segm:Offs+I2]));
          IF Test='N5NX' THEN TFPCR_INT:=I1;
          INC(I2);
        END;
      TFPCRInstall:=(TFPCR_INT<>0); 
    END;
END;


{-------------------------------------------------------------------------}
{ TFPCR Datenausgabe                                                      }
{Globale Variablen:                                                       }
{CPUREG    :Registerrecord fuer INTR-Prozedur                             }
{V24Init[] :Installationsstatus des TFPCX-Port                            }
{-------------------------------------------------------------------------}

Procedure TX_TFPCR(Ausgabe:Byte);

BEGIN
  IF V24Init[5] THEN
    BEGIN
      CPUREG.AH:=3;                {AH-Register ist Ausgabekennung hier 3}
      CPUREG.AL:=Ausgabe;          {AL-Refister ist auszugebendes Byte}
      INTR(TFPCR_INT,CPUREG);
    END;  
END;


{-------------------------------------------------------------------------}
{ TFPCR-Daten und Statusempfangsinterface                                 }
{Globale Variablen:                                                       }
{CPUREG    :Registerrecord fuer INTR-Prozedur                             }
{V24Init[] :Installationsstatus des TFPCX-Port                            }
{-------------------------------------------------------------------------}

Function RX_TFPCR(Funktion:Byte):Byte;

BEGIN
  IF V24Init[5] THEN
    BEGIN
      CPUREG.AH:=Funktion;   {AH-Register ist Funktion 1=Empfangsstatus}
      INTR(TFPCR_INT,CPUREG);{                         2=Zeichen empfangen}
      RX_TFPCR:=CPUREG.AL;   {bei 1: 0=keine Zeichen sonst 1 Zeichen}
    END;                     {bei 2: Zeichencode}
END;


{=========================================================================}
{Procedure V24DefHW ***UNIT-Interface***                                  }
{Diese Prozedur erlaubt eine Hardwaremaessige Umdefinition der als Typ-   }
{konstante in der V24 UNIT vordefinierten Hardwaredaten COM-Port Basis-   }
{adresse und IRQ zur Anpassung an exotische COM3/4-Schnittstellen         }
{Interfacevariablen                                                       }
{SER           -Schnittstellennummer COMX (1-5)                           }
{Basis         -Basisadresse                                              }
{COMPort[SER]  -Basisadressdefinition der UNIT                            }
{IRQ           -IRQ der Karte                                             }
{COMIRQ[SER]   -IRQ-Definition der UNIT                                   }
{=========================================================================}

Procedure V24DefHW;

BEGIN
  IF SER=5 THEN EXIT;      {Nicht beim TFPCR-Interface!}
  COMPort[SER]:=Basis;
  COMIRQ[SER] :=IRQ;
END;


{=========================================================================}
{Procedure V24DefFCR ***UNIT-Interface***                                 }
{Diese Prozedur erlaubt eine Benutzerdefinition des FCR-Konfigurations-   }
{byte.                                                                    }
{Interfacevariable:                                                       }
{V24FCRData     Uebergebenes FCR-Byte                                     }
{SER            Schnittstellennummer COMX (1-5)                           }
{Globale Variable:                                                        }
{FCRByte[]      Interne FCR Konfigurationsbytes der COM-Ports             }
{=========================================================================}

Procedure V24DefFCR;

BEGIN
  IF SER=5 THEN EXIT;                 {Nicht beim TFPCR-Interface!}
  FCRByte[SER]:=FCRData;
  FCRByte[SER]:=FCRByte[SER] AND $C1; {Nur Bit 0 und Bits 6-7 verwenden}   
END;


{=========================================================================}
{Procedure SetRTS   ***UNIT-Interface***                                  }
{Diese Procedure setzt die Handshakeleitung RTS auf log. 1                }
{Interfacevariable:                                                       }
{SER         Schnittstellennummer 1-4                                     }
{Globale Variablen:                                                       }
{COMPort[]     Basisadresse COM1-COM4                                     }
{MCROff        Registeroffset Modemcontrolregister                        }
{RTSMask       Bitmaske RTS im Modemcontrolregister                       }  
{=========================================================================}

Procedure SetRTS;

BEGIN
  IF SER=5 THEN EXIT;  {Nicht beim TFPCR-SW-Interface}
  PORT[COMPort[SER]+MCROff]:=(PORT[COMPort[SER]+MCROff] OR RTSMask);
END;


{=========================================================================}
{Procedure ClearRTS   ***UNIT-Interface***                                }
{Diese Procedure setzt die Handshakeleitung RTS auf log. 0                }
{Interfacevariable:                                                       }
{SER             Schnittstellennummer 1-4                                 }
{Globale Variablen:                                                       }
{COMPort[SER]    Basisadresse COM1-COM4                                   }
{MCROff          Registeroffset Modemcontrolregister                      }
{RTSMask         Bitmaske RTS im Modemcontrolregister                     }  
{=========================================================================}

Procedure ClearRTS;

BEGIN
  IF SER=5 THEN EXIT;  {Nicht beim TFPCR-SW-Interface}
  PORT[COMPort[SER]+MCROff]:=(PORT[COMPort[SER]+MCROff] AND (255-RTSMask));
END;


{=========================================================================}
{Procedure SetDTR   ***UNIT-Interface***                                  }
{Diese Procedure setzt die Handshakeleitung DTR auf log. 1                }
{Globale Variablen:                                                       }
{COMPort[SER]    Basisadresse COM1-COM4                                   }
{MCROff          Registeroffset Modemcontrolregister                      }
{DTRMask         Bitmaske DTR im Modemcontrolregister                     }  
{=========================================================================}

Procedure SetDTR;

BEGIN
  IF SER=5 THEN EXIT;  {Nicht beim TFPCR-SW-Interface}
  PORT[COMPort[SER]+MCROff]:=(PORT[COMPort[SER]+MCROff] OR DTRMask);
END;


{=========================================================================}
{Procedure ClearDTR   ***UNIT-Interface***                                }
{Diese Procedure setzt die Handshakeleitung RTS auf log. 0                }
{Globale Variablen:                                                       }
{COMPort[SER]    Basisadresse COM1-COM4                                   }
{MCROff          Registeroffset Modemcontrolregister                      }
{DTRMask         Bitmaske DTR im Modemcontrolregister                     }  
{=========================================================================}

Procedure ClearDTR;

BEGIN
  IF SER=5 THEN EXIT;  {Nicht beim TFPCR-SW-Interface}
  PORT[COMPort[SER]+MCROff]:=(PORT[COMPort[SER]+MCROff] AND (255-DTRMask));
END;


{=========================================================================}
{Function V24Count  ***UNIT-Interface***                                  }
{Uebergibt die Anzahl Zeichen im RS232-Puffer an das Anwenderprogramm     }
{Bei TFPCx uebernimmt diese Routine die Funktion des Empfangshandlers und }
{traegt die Zeichen in den Interruptpuffer ein.                           }
{Globale Variable:                                                        }
{Puffer[]      Zeichenzahl im Empfasngspuffer vom COMX                    }
{INTZEI[]      Schreibzeiger auf Interruptzeichenpuffer                   }
{LESZEI[]      Lesezeiger auf Interruptzeichenpuffer                      }
{Overrun[]     Ueberlaufstatus des COMx-Ringpuffers                       }
{SER           Schnittstellennummer fuer TFPCx-Bearbeitung                }
{=========================================================================}

Function V24Count;

BEGIN
  IF SER=5 THEN
    BEGIN
      IF (RX_TFPCR(1)<>0) THEN
        BEGIN
          V24Puffer[5,INTZEI[5]]:=RX_TFPCR(2);
          IF INTZEI[5] >= SIZE THEN INTZEI[5]:= 1 ELSE Inc(INTZEI[5]);
          Inc (Puffer[5]);
          {Bei Pufferueberlauf enthaelt Ringpuffer die letzten SIZE Zeichen}
          IF Puffer[5]>SIZE THEN
            BEGIN
              IF LESZEI[5]>=SIZE THEN LESZEI[5]:=1 ELSE INC (LESZEI[5]);
              Puffer[5]:=SIZE;
              Overrun[5]:=TRUE;
            END
          ELSE Overrun[5]:=False;
        END;
    END
  ELSE
    BEGIN
      {Im Empfangsregister oder 16550 RX-FIFO vorhandene Zeichen lesen}
      Inline ($FA);   {Keine Interrupts waehrend dem Auslesen des UART}
      While (Port[COMPort[SER]+LSRoff] AND RDRMask)=RDRMask DO
        BEGIN
          {Nachlauf-Ringpuffer}
          V24Puffer[SER,INTZEI[SER]]:=Port[COMPort[SER]];
          IF INTZEI[SER]>=SIZE THEN INTZEI[SER]:=1 ELSE Inc(INTZEI[SER]);
          Inc (Puffer[SER]);
          {Bei Pufferueberlauf enthaelt Ringpuffer die letzten SIZE Zeichen}
          IF Puffer[SER]>SIZE THEN
            BEGIN
                IF LESZEI[SER]>=SIZE THEN LESZEI[SER]:=1 ELSE INC (LESZEI[SER]);
              Puffer[SER]:=SIZE;
              Overrun[SER]:=TRUE;
            END
          ELSE Overrun[SER]:=False;
        END;
      Inline ($FB);   {Interrupts wieder freigeben}
    END;
  V24Count:=Puffer[SER];
END;


{=========================================================================}
{Function V24Aktive  ***UNIT-Interface***                                 }
{Uebergibt dem Anwenderprogramm den Status ob der Schnittstellenhandler   }
{installiert ist oder nicht                                               }
{Globale Variable:                                                        }
{V24INIT[]     Status ob COMX initialisiert ist oder nicht                }
{=========================================================================}

Function V24Aktive;

BEGIN
  V24Aktive:=V24Init[SER];
END;


{=========================================================================}
{Function V24Overrun  ***UNIT-Interface                                   }
{Uebergibt dem Anwenderprogramm den Ueberlaufstatus des RX-Ringpuffers    }
{Globale Variable:                                                        }
{Overrun[]    Status ob Ringpufferueberlauf vorliegt TRUE=Ja FALSE=Nein   }
{=========================================================================}

Function V24Overrun;

BEGIN
  V24Overrun:=Overrun[SER];
END;
 

{=========================================================================}
{Function TXReady  ***UNIT-Interface***                                   }
{Uebergibt den Handshakestatus der Sendeschnittstelle an das Anwender-    }
{programm                                                                 } 
{Globale Variablen:                                                       }
{TXHand[]        Sendehandshakestatus von COMx                            }
{Hard[]          Status fuer Hardwarehandshake an COMX                    }
{Soft[]          Status fuer Softwarehandshake an COMX                    }
{COMPort[SER]  Basisadresse COM1-COM4                                     }
{MSROff          Registeroffset Modemstatusstatusregister                 }
{DSRMask         Bitmaske DTR im Modemstatusregister                      }  
{CTSMask         Bitmaske CTS im Modemstatusregister                      } 
{SER           serielle Schnittstelle 1-4 und 5 fuer TFPCx Interface      }
{=========================================================================}

Function TXReady;

BEGIN
  IF Hard[SER] AND (SER<>5) THEN TXHand[SER]:=((PORT[COMPort[SER]+MSRoff] AND
                                               (DSRMask+CTSMask))
                                             =(DSRMask+CTSMask));
  IF Soft[SER] AND (SER=5) THEN TXHand[SER]:=(RX_TFPCR($FD)<>0);
  TXReady:=TXHand[SER];
END;


{=========================================================================}
{Function RXReady  ***UNIT-Interface***                                   }
{Uebergibt den Handshakestatus der Empfangsschnittstelle an das Anwender- }
{programm                                                                 }
{Globale Variable:                                                        }
{RXHAND[]      Empfangshandshakestatus von COMX                           }
{=========================================================================}

Function RXReady;

BEGIN
  RXReady:=RXHand[SER];
END;


{=========================================================================}
{Function ReadCTS  ***UNIT-Interface***                                   }
{Liest den Status der Handshakeleitung CTS aus. Der Wert 16 bedeutet das  }
{CTS gesetzt ist und 0 bedeutet das CTS nicht gesetzt ist.                }
{Globale Variablen:                                                       }
{SER             Nummer der seriellen Schnittstelle 1-4                   }
{COMPort[SER]    Basisadresse von COMX                                    }
{MSROff          Registeroffset Modemstatusregister                       }
{CTSMask         Bitmaske fuer CTS-Bit im Modemstatusregister             }
{=========================================================================}

Function ReadCTS;

BEGIN
  IF SER=5 THEN EXIT;  {Nicht beim TFPCR-SW-Interface}
  ReadCTS:=Port[COMPort[SER]+MSRoff] AND CTSMask;
END;


{=========================================================================}
{Function ReadDSR  ***UNIT-Interface***                                   }
{Liest den Status der Handshakeleitung DSR aus. Der Wert 32 bedeutet das  }
{DSR gesetzt ist und 0 bedeutet das DSR nicht gesetzt ist.                }
{Globale Variablen:                                                       }
{SER             Nummer der seriellen Schnittstelle 1-4                   }
{COMPort[SER]    Basisadresse von COMX                                    }
{MSROff          Registeroffset Modemstatusregister                       }
{DSRMask         Bitmaske fuer DSR-Bit im Modemstatusregister             }
{=========================================================================}

Function ReadDSR;

BEGIN
  IF SER=5 THEN EXIT;  {Nicht beim TFPCR-SW-Interface}
  ReadDSR:=Port[COMPort[SER]+MSRoff] AND DSRMask;
END;


{=========================================================================}
{Function ReadDCD  ***UNIT-Interface***                                   }
{Liest den Status der Statusleitung DCD aus. Der Wert 128 bedeutet das    }
{DCD gesetzt ist und 0 bedeutet das DCD nicht gesetzt ist.                }
{Globale Variablen:                                                       }
{SER             Nummer der seriellen Schnittstelle 1-4                   }
{COMPort[SER]    Basisadresse von COMX                                    }
{MSROff          Registeroffset Modemstatusregister                       }
{DCDMask (128)   Bitmaske fuer DCD-Bit im Modemstatusregister             }
{=========================================================================}

Function ReadDCD;

BEGIN
  IF SER=5 THEN EXIT;  {Nicht beim TFPCR-SW-Interface}
  ReadDCD:=Port[COMPort[SER]+MSRoff] AND DCDMask;
END;


{=========================================================================}
{Function ReadRI  ***UNIT-Interface***                                    }
{Liest den Status der Statusleitung RI aus. Der Wert 64 bedeutet das      }
{RI gesetzt ist und 0 bedeutet das RI nicht gesetzt ist.                  }
{Globale Variablen:                                                       }
{SER             Nummer der seriellen Schnittstelle 1-4                   }
{COMPort[SER]    Basisadresse von COMX                                    }
{MSROff          Registeroffset Modemstatusregister                       }
{RIMask          Bitmaske fuer RI-Bit im Modemstatusregister              }
{=========================================================================}

Function ReadRI;

BEGIN
  IF SER=5 THEN EXIT;  {Nicht beim TFPCR-SW-Interface}
  ReadRI:=Port[COMPort[SER]+MSRoff] AND RIMask;
END;


{=========================================================================}
{Function V24Install  ***UNIT-Interface***                                }
{berprueft durch Abfrage der fuer das LSR typischen Maske 060H und dem   }
{Inhalt des IIR (immer <=7) ob eine Schnittstelle installiert ist.        }
{Der Test erfolgt im 8250/16450 Modus sofern ein 16550 installiert ist.   }
{Bei COM5 TFPCR-Interface wird ueberprueft ob der Treiber installiert ist }
{Interne Konstanten:                                                      }
{LSRMask         :Bitmaske zur Identifizierung des LSR                    }
{IIRMAX          :Groesster Wert zur Identifiezierung des IIR             }
{Globale Variablen und Konstanten:                                        }   
{Test            :Schnittstellennummer COM"X"                             }
{COMPort[Test]   :Basisadresse COMX                                       }
{LSROff          :Registeroffset Line Status Register                     }
{IIROff          :Registeroffset Interrupt Identification Register        }
{FCRoff          :Registeroffset FIFO Control Register des 16550          }
{FCRByte[]       :FIFO Control Register Konfigurationsbytes               }
{INHLSR          :Zwischenspeicher Line Status Register                   }
{INHIIR          :Zwischenspeicher Interrupt Identification Register      } 
{=========================================================================}

Function V24Install;

CONST LSRMSK  =$60;   {Bitmaske fuer Line Statusregister}
      IIRMAX  =$07;   {Maximaler Wert im Interrupt Identification Register}

VAR   INHLSR:      BYTE;  {IO-Inhalt LSR-Register}
      INHIIR:      BYTE;  {IO-Inhalt IIR-Register}

BEGIN
  IF Test=5 THEN V24Install:=TFPCRInstall          {V24Install=TFPCRInstall}
  ELSE
    BEGIN
      INHLSR:=Port[COMPort[Test]+LSROFF];
      {1. Suchbedingung: Line-Status-Reg. muss Bitmuster $60H enthalten.}
      IF ((INHLSR AND LSRMSK)=LSRMSK) AND (INHLSR<>$FF) THEN
        BEGIN
          {16550 UART in den 16450 Modus zurueckschalten}
          Port[COMPort[Test]+FCRoff]:=0;
          INHIIR:=Port[ComPort[Test]+IIROFF];
          {2. Suchbedingung: Interrupt Identification Register muss <=7 sein.}
          IF (INHIIR <=IIRMAX) THEN V24Install:=TRUE; {Hardware vorhanden}
          {16550 UART in den FIFO-Modus zurueckschalten}
          Port[COMPort[Test]+FCRoff]:=FCRByte[Test];
        END
      ELSE V24Install:=FALSE;                      {Hardware nicht vorhanden}
    END;
END;


{=========================================================================}
{Procedure V24SetTXR  ***UNIT-Interface***                                }
{Diese Prozedur setzt den Status TXHand auf TRUE um die Aussendung von    }
{Zeichen trotz Handshakesperre der Sendeschnittstelle zu ermoeglichen.    }
{Diese Prozedur dient zum Freischalten der UNIT bei undefiniertem Verhal- }
{ten des angeschlossenen Endgeraetes.                                     }
{=========================================================================}

Procedure V24SetTXR;

BEGIN
  TXHand[SER]:=TRUE;
END;


{=========================================================================}
{Procedure SendBreak  ***UNIT-Interface***                                }
{Diese Procedure aktiviert den Breakzustand ueber das LCR fuer "Zeit" ms  }
{als BREAK-Signalinterrupt                                                }
{Interne Variablen und Konstanten:                                        }
{BreakMask:  Bit 6 des Linecontrolregisters zur Breaksteuerung 1=Break an }
{AktAttr  :  Zwischenspeicher fuer das aktuelle Textattributbit           }
{Globale Variablen und Konstanten:                                        }
{SER           :Schnittstellennummer COM"X"                               }
{COMPort[]     :Basisadresse COMX                                         }
{LCRoff        :Registeroffset Line Control Register                      }
{=========================================================================}

Procedure SendBreak;

CONST   BreakMask =64;       {Break Aktivierungsbit im LCR}

BEGIN
  IF SER=5 THEN EXIT         {Nicht bei TFPCR/TFPCX moeglich}
  ELSE
    BEGIN
      Port[ComPort[SER]+LCRoff]:=Port[ComPort[SER]+LCRoff] OR BreakMask;
      Delay(Zeit);
      Port[ComPort[SER]+LCRoff]:=Port[ComPort[SER]+LCRoff] AND (255-BreakMask);
    END;
END;


{=========================================================================}
{Procedure V24Write  ***UNIT-Interface***                                 }
{Diese Procedure gibt ein Zeichen auf der Schnittstelle aus nachdem der   }
{Handshakestatus getestet wurde. Liegt ein Sperrstatus vor so wird das    }
{Zeichen nicht gesendet und die globale Variable TXHand auf FALSE         }
{ermittelt.                                                               }
{Bei Baudrate 0 wird das Zeichen auf das TFPCR-Softwareinterface          }
{ausgegeben.                                                              }
{Konstanten und Variablen:                                                }
{TXEMP         :Pruefmaske fuer Transmitter Register Empty-Status         }
{Globale Variablen und Konstanten                                         }
{BAUD          :Indikator fuer TFPCR-Software-Interface                   } 
{TXHand[]      :Pruefung auf Handshakesperre                              }
{SER           :Schnittstellennummer COM"X"                               }
{COMPort[]     :Basisadresse COMX                                         }
{LSRoff        :Registeroffset Line Status Register                       }
{Hard[]        :Statusbit fuer Handshake TRUE=HW-Handshake aktiv          }
{=========================================================================}

PROCEDURE V24Write;

CONST   TXEMP =32;      {Bitmaske fuer Transmitter Register Empty}

BEGIN
  {TFPCR-Software-Interface}
  IF (SER=5) AND TXHand[SER] THEN
    BEGIN
      TX_TFPCR(Ord(Zeichen));
      EXIT;
    END;

  IF Hard[SER] THEN TXHand[SER]:=((ReadCTS(SER)<>0) AND (ReadDSR(SER)<>0));
  IF TXHand[SER] THEN
    BEGIN
      REPEAT UNTIL (PORT[COMPort[SER]+LSRoff] AND TXEMP) = TXEMP;
      PORT[COMPort[SER]]:= Ord(Zeichen);
    END;
END;


{=========================================================================}
{Function V24Read  ***UNIT-Interface***                                   }
{Diese Function liesst ein Zeichen aus dem Interruptpuffer aus.           }
{Bei COM5 wird das Zeichen vom TFPCx-Softwareinterface gelesen.           }
{Globale Konstanten und Variablen:                                        }
{LESZEI[]            :Index auf Pufferfeld                                }
{V24Puffer[LESZEI[]] :RS232-Ringpufferfeld                                }
{Puffer[]            :Anzahl noch ausstehender Zeichen im Ringpuffer      }
{Size                :Groesse des Ringpuffers                             }
{RXHand[]            :Eigener Handshakestatus bei Sperre = FALSE          }
{                     wird von Interruptroutine gesperrt hier freigegeben }
{                     wenn Puffer=0                                       }
{SER                 :Serielle Schnittstelle 5 ist TFPCx Soft Interface   }
{Overrun[]           :Ueberlaufstatus des Ringpuffers                     }
{=========================================================================}

FUNCTION V24Read;

BEGIN
  {Ueberpruefung des Overrunstatus (Ringpufferueberlauf)}
  IF Puffer[SER]<=SIZE THEN Overrun[SER]:=False;

  {TFPCR-Software-Interface}
  IF SER=5 THEN
    BEGIN
      IF (Puffer[SER]>0) THEN
        BEGIN
          V24Read:=Chr(V24Puffer[SER,LESZEI[SER]]);
          IF LESZEI[SER] >= SIZE THEN LESZEI[SER]:= 1 ELSE Inc(LESZEI[SER]);
          Dec (Puffer[SER]);
        END;
      EXIT;
    END;

  {Normales UART-Interface COM1-COM4}
  Inline($FA);  { CLI keine Interrupts beim Auslesen des Puffers}
  IF (Puffer[SER]>0) THEN
    BEGIN
      {Ringpuffernachlauf}
      V24Read := Chr(V24Puffer[SER,LESZEI[SER]]);
      IF LESZEI[SER] >= SIZE THEN LESZEI[SER]:= 1 ELSE Inc(LESZEI[SER]);
      Dec (Puffer[SER]);
    END;
  IF (RXHand[SER]=false) AND (Puffer[SER]=0) THEN
    BEGIN
      IF Soft[SER] THEN V24Write(SER,Chr(XON));
      IF Hard[SER] THEN SetRTS(SER);
      RXHand[SER]:=TRUE;
    END;
  Inline($FB);  { sti }
END;


{-------------------------------------------------------------------------}
{Procedure SIOClear                                                       }
{Ruecksetzen eventuell vorhandener Fehlerzustaende/Interrupts/Restzeichen }
{um UART-Hardware Fehler (Haengenbleiben des UART) zu verhindern.         }
{Interne Variablen und Konstanten:                                        }
{SER             :Schnittstellennummer COM"X"                             }
{RLSINT          :Bitmaske fuer Line Status Interrupt im IIR              }
{RDAINT          :Bitmaske fuer Receiver Data Available Interrupt im IIR  }
{LSRRDY          :Line Status Register bei sauber abgefragtem und berei-  }
{                 tem UART Bits 5,6 zeigen leere Senderegister an         }
{Dummi           :Hilfsvariable zum Registerauslesen und Zwischenspeichern}
{OldIER          :Zwischenspeicher fuer Interrupt Enable Register         }
{Tries           :Anzahl Ruecksetzversuche unerlaubter Zustaende des UART }
{Count           :Anzahl Ruecksetzversuche in den Kontrollschleifen       }
{Globale Variablen und Konstanten                                         }
{COMPort[]       :Basisadresse COMX                                       }
{LSRoff          :Registeroffset Line Status Register                     }
{IIRoff          :Registeroffset Interrupt Identification Register        }
{IERoff          :Registeroffset Interrupt Enable Register                }
{IntCont         :Interruptcontroller 1 oder 2                            }
{Port8259[]      :Basisadressen und EOI-Kontrollregister der 8259         }
{EOI8259         :Bitmaske fuer Non Specific Interrupt Ende 8259          }
{-------------------------------------------------------------------------}


Procedure SIOClear(SER:Byte);

CONST   RLSINT=06;           {Maske zur Erkennung Receiver Line Status-INT}
        RDAINT=04;           {Maske zur Erkennung Receiver Data Status-INT}
        LSRRDY=$60;          {LSR im Ruhezustand -nur TXR, THR Empty High-}
        Tries =10;           {Maximal 10 Ruecksetzversuche }

VAR     Dummi :Byte;         {Hilfsvariable zum Registerlesen und sichern }
        OldIER:Byte;         {Zwischenspeicher fuer Interrupt Enable Register}
        Count :Byte;         {Anzahl Ruecksetzversuche bis Abbruch}

BEGIN
  {Wenn gewuenschter COM-Port nicht installiert sofort abbrechen}
  IF V24Install(SER)=False THEN EXIT;

  {Overrun-ZUSTAND loeschen}
  Count:=1;
  While (Port[ComPort[SER]+LSRoff] <> LSRRDY) AND (Count <= Tries*16) DO
    BEGIN
      Dummi:=Port[ComPort[SER]];
      Inc(Count);
    END;

  {Beim Loeschen der alten Interruptzustaende neue Interrupts sperren}
  OldIER:=Port[ComPort[SER]+IERoff];
  Port[ComPort[SER]+IERoff]:=0;

  {Eventuell unbearbeitete Interruptzustaende des 8250 UART loeschen}
  Count:=1;
  While (Port[ComPort[SER]+IIRoff]=0) AND (Count<=Tries) DO
    BEGIN
      {Line Status-Interrupt durch Auslesen des LSR loeschen}
      IF (Port[ComPort[SER]+IIRoff] AND RLSINT)=RLSINT THEN
        BEGIN
          Dummi:=Port[ComPort[SER]+LSRoff];
          Port[Port8259[1]]:=EOI8259;
          IF IntCont=2 THEN Port[Port8259[2]]:=EOI8259;
        END;
      {Receiver Data-Interrupt durch Auslesen des Empfangsregisters loeschen}
      IF (Port[ComPort[SER]+IIRoff] AND RDAINT)=RDAINT THEN
        BEGIN
          Dummi:=Port[ComPort[SER]];
          Port[Port8259[1]]:=EOI8259;
          IF IntCont=2 THEN Port[Port8259[2]]:=EOI8259;
        END;
      {Modem-Status-INT durch Auslesen des MSR loeschen}
      IF Port[ComPort[SER]+IIRoff]=0 THEN
        BEGIN
          Dummi:=Port[ComPort[SER]+MSRoff];
          Port[Port8259[1]]:=EOI8259;
          IF IntCont=2 THEN Port[Port8259[2]]:=EOI8259;
        END;
      {Ueberwachungszaehler inkrementieren}
      Inc(Count);
    END;

  {Altes IER vorm Beenden zurueckspeichern}
  Port[ComPort[SER]+IERoff]:=OldIER;
END;


{-------------------------------------------------------------------------}
{Procedure V24RingInt1                                                    }
{Diese Procedure ist eine einfache aber schnelle Ringpuffer-Interrupt     }
{Routine fuer Anwendungen ohne Handshakekontrolle durch die Interrupt-    }
{Routine                                                                  }
{Globale Konstanten und Variable:                                         }
{INTZEI[]            :Schreibzeiger auf Ringpufferfeld                    }
{LESZEI[]            :Lesezeiger auf Ringpufferfeld                       }
{V24Puffer[INTZEI[]] :RS232-Ringpufferfeld                                }
{Puffer[]            :Anzahl noch ausstehender Zeichen im Ringpuffer      }
{Size                :Groesse des Ringpuffers                             }
{COMPort[]           :Basisadresse COMX                                   }
{IntCont             :Interruptcontroller 1 oder 2                        }
{Port8259[]          :Basisadressen und EOI-Kontrollregister der 8259     }
{EOI8259             :Bitmaske fuer Non Specific Interrupt Ende 8259      }
{LSRoff              :Registeroffset Line Status Register                 }
{RDRMask             :Receiver Data Ready Bit im Line Status Register     }
{Overrun[]           :Ueberlaufstatus des Empfangsringpuffers             }
{-------------------------------------------------------------------------}

(*$F+*)
PROCEDURE V24RingInt1;
(*$F-*)
Interrupt;

BEGIN
  Inline ($FA);         {Interruptroutine darf nicht unterbrochen werden}
  
  {Bei 8250/16450 1 Zeichen, bei 16550 alle Zeichen im RX-FIFO einlesen}
  While (Port[COMPort[1]+LSRoff] AND RDRMask)=RDRMask DO
    BEGIN
      {Nachlauf-Ringpuffer}
      V24Puffer[1,INTZEI[1]] := Port[COMPort[1]];
      IF INTZEI[1]>=SIZE THEN INTZEI[1]:=1 ELSE Inc(INTZEI[1]);
      Inc (Puffer[1]);
      {Bei Pufferueberlauf enthaelt Ringpuffer die letzten SIZE Zeichen}
      IF Puffer[1]>SIZE THEN
        BEGIN
          IF LESZEI[1]>=SIZE THEN LESZEI[1]:=1 ELSE INC (LESZEI[1]);
          Puffer[1]:=SIZE;
          Overrun[1]:=TRUE;
        END
      ELSE Overrun[1]:=False;
    END;

  {Non spec. EOI fuer 2. 8259 bei IRQ 8-15}
  IF IntCont=2 THEN Port[Port8259[2]]:=EOI8259;

  {Non spec. EOI fuer 1. 8259 bei IRQ 1-7}
  Port[Port8259[1]]:=EOI8259;

  Inline ($FB);         {Interrupts wieder freigeben}
END;


{-------------------------------------------------------------------------}
{Procedure V24RingInt2                                                    }
{Diese Procedure ist eine einfache aber schnelle Ringpuffer-Interrupt     }
{Routine fuer Anwendungen ohne Handshakekontrolle durch die Interrupt-    }
{Routine                                                                  }
{Globale Konstanten und Variable:                                         }
{INTZEI[]            :Schreibzeiger auf Ringpufferfeld                    }
{LESZEI[]            :Lesezeiger auf Ringpufferfeld                       }
{V24Puffer[INTZEI[]  :RS232-Ringpufferfeld                                }
{Puffer[]            :Anzahl noch ausstehender Zeichen im Ringpuffer      }
{Size                :Groesse des Ringpuffers                             }
{COMPort[]           :Basisadresse COMX                                   }
{IntCont             :Interruptcontroller 1 oder 2                        }
{Port8259[]          :Basisadressen und EOI-Kontrollregister der 8259     }
{EOI8259             :Bitmaske fuer Non Specific Interrupt Ende 8259      }
{LSRoff              :Registeroffset Line Status Register                 }
{RDRMask             :Receiver Data Ready Bit im Line Status Register     }
{Overrun[]           :Ueberlaufstatus des Empfangsringpuffers             }
{-------------------------------------------------------------------------}

(*$F+*)
PROCEDURE V24RingInt2;
(*$F-*)
Interrupt;

BEGIN
  Inline ($FA);         {Interruptroutine darf nicht unterbrochen werden}
  
  {Bei 8250/16450 1 Zeichen, bei 16550 alle Zeichen im RX-FIFO einlesen}
  While (Port[COMPort[2]+LSRoff] AND RDRMask)=RDRMask DO
    BEGIN
      {Nachlauf-Ringpuffer}
      V24Puffer[2,INTZEI[2]] := Port[COMPort[2]];
      IF INTZEI[2]>=SIZE THEN INTZEI[2]:=1 ELSE Inc(INTZEI[2]);
      Inc (Puffer[2]);
      {Bei Pufferueberlauf enthaelt Ringpuffer die letzten SIZE Zeichen}
      IF Puffer[2]>SIZE THEN
        BEGIN
          IF LESZEI[2]>=SIZE THEN LESZEI[2]:=1 ELSE INC (LESZEI[2]);
          Puffer[2]:=SIZE;
          Overrun[2]:=TRUE;
        END
      ELSE Overrun[2]:=False;
    END;

  {Non spec. EOI fuer 2. 8259 bei IRQ 8-15}
  IF IntCont=2 THEN Port[Port8259[2]]:=EOI8259;

  {Non spec. EOI fuer 1. 8259 bei IRQ 1-7}
  Port[Port8259[1]]:=EOI8259;

  Inline ($FB);         {Interrupts wieder freigeben}
END;


{-------------------------------------------------------------------------}
{Procedure V24RingInt3                                                    }
{Diese Procedure ist eine einfache aber schnelle Ringpuffer-Interrupt     }
{Routine fuer Anwendungen ohne Handshakekontrolle durch die Interrupt-    }
{Routine                                                                  }
{Globale Konstanten und Variable:                                         }
{INTZEI[]            :Schreibzeiger auf Ringpufferfeld                    }
{LESZEI[]            :Lesezeiger auf Ringpufferfeld                       }
{V24Puffer[INTZEI[]] :RS232-Ringpufferfeld                                }
{Puffer[]            :Anzahl noch ausstehender Zeichen im Ringpuffer      }
{Size                :Groesse des Ringpuffers                             }
{COMPort[]           :Basisadresse COMX                                   }
{IntCont             :Interruptcontroller 1 oder 2                        }
{Port8259[]          :Basisadressen und EOI-Kontrollregister der 8259     }
{EOI8259             :Bitmaske fuer Non Specific Interrupt Ende 8259      }
{LSRoff              :Registeroffset Line Status Register                 }
{RDRMask             :Receiver Data Ready Bit im Line Status Register     }
{Overrun[]           :Ueberlaufstatus des Empfangsringpuffers             }
{-------------------------------------------------------------------------}

(*$F+*)
PROCEDURE V24RingInt3;
(*$F-*)
Interrupt;

BEGIN
  Inline ($FA);         {Interruptroutine darf nicht unterbrochen werden}
  
  {Bei 8250/16450 1 Zeichen, bei 16550 alle Zeichen im RX-FIFO einlesen}
  While (Port[COMPort[3]+LSRoff] AND RDRMask)=RDRMask DO
    BEGIN
      {Nachlauf-Ringpuffer}
      V24Puffer[3,INTZEI[3]] := Port[COMPort[3]];
      IF INTZEI[3]>=SIZE THEN INTZEI[3]:=1 ELSE Inc(INTZEI[3]);
      Inc (Puffer[3]);
      {Bei Pufferueberlauf enthaelt Ringpuffer die letzten SIZE Zeichen}
      IF Puffer[3]>SIZE THEN
        BEGIN
          IF LESZEI[3]>=SIZE THEN LESZEI[3]:=1 ELSE INC (LESZEI[3]);
          Puffer[3]:=SIZE;
          Overrun[3]:=TRUE;
        END
      ELSE Overrun[3]:=False;
    END;

  {Non spec. EOI fuer 2. 8259 bei IRQ 8-15}
  IF IntCont=2 THEN Port[Port8259[2]]:=EOI8259;

  {Non spec. EOI fuer 1. 8259 bei IRQ 1-7}
  Port[Port8259[1]]:=EOI8259;

  Inline ($FB);         {Interrupts wieder freigeben}
END;


{-------------------------------------------------------------------------}
{Procedure V24RingInt4                                                    }
{Diese Procedure ist eine einfache aber schnelle Ringpuffer-Interrupt     }
{Routine fuer Anwendungen ohne Handshakekontrolle durch die Interrupt-    }
{Routine                                                                  }
{Globale Konstanten und Variable:                                         }
{INTZEI[]            :Schreibzeiger auf Ringpufferfeld                    }
{LESZEI[]            :Lesezeiger auf Ringpufferfeld                       }
{V24Puffer[INTZEI[]] :RS232-Ringpufferfeld                                }
{Puffer[]            :Anzahl noch ausstehender Zeichen im Ringpuffer      }
{Size                :Groesse des Ringpuffers                             }
{COMPort[]           :Basisadresse COMX                                   }
{IntCont             :Interruptcontroller 1 oder 2                        }
{Port8259[]          :Basisadressen und EOI-Kontrollregister der 8259     }
{EOI8259             :Bitmaske fuer Non Specific Interrupt Ende 8259      }
{LSRoff              :Registeroffset Line Status Register                 }
{RDRMask             :Receiver Data Ready Bit im Line Status Register     }
{Overrun[]           :Ueberlaufstatus des Empfangsringpuffers             }
{-------------------------------------------------------------------------}

(*$F+*)
PROCEDURE V24RingInt4;
(*$F-*)
Interrupt;

BEGIN
  Inline ($FA);         {Interruptroutine darf nicht unterbrochen werden}
  
  {Bei 8250/16450 1 Zeichen, bei 16550 alle Zeichen im RX-FIFO einlesen}
  While (Port[COMPort[4]+LSRoff] AND RDRMask)=RDRMask DO
    BEGIN
      {Nachlauf-Ringpuffer}
      V24Puffer[4,INTZEI[4]] := Port[COMPort[4]];
      IF INTZEI[4]>=SIZE THEN INTZEI[4]:=1 ELSE Inc(INTZEI[4]);
      Inc (Puffer[4]);
      {Bei Pufferueberlauf enthaelt Ringpuffer die letzten SIZE Zeichen}
      IF Puffer[4]>SIZE THEN
        BEGIN
          IF LESZEI[4]>=SIZE THEN LESZEI[4]:=1 ELSE INC (LESZEI[4]);
          Puffer[4]:=SIZE;
          Overrun[4]:=TRUE;
        END
      ELSE Overrun[4]:=False;
    END;

  {Non spec. EOI fuer 2. 8259 bei IRQ 8-15}
  IF IntCont=2 THEN Port[Port8259[2]]:=EOI8259;

  {Non spec. EOI fuer 1. 8259 bei IRQ 1-7}
  Port[Port8259[1]]:=EOI8259;

  Inline ($FB);         {Interrupts wieder freigeben}
END;


{-------------------------------------------------------------------------}
{Procedure V24HSInt1                                                      }
{Diese Procedure ist eine Ringpuffer-Interrupt Routine mit automatischer  }
{Handshakesteuerung der Empfangsschnittstelle und Setzen des Sperrhand-   }
{shake bei drohendem Pufferueberlauf                                      }
{Bei Softwarehandshake Setzen des eigenen Sendestatus TXHand und Aus-     }
{filtern von XON und XOFF                                                 }
{Interne Konstanten:                                                      }
{Reserve             :Ausloeseabstand zum Pufferende fuer Handshakesperre }
{Globale Konstanten und Variable:                                         }
{INTZEI[]            :Schreibzeiger auf Ringpufferfeld                    }
{LESZEI[]            :Lesezeiger auf Ringpufferfeld                       }
{V24Puffer[INTZEI[]] :RS232-Ringpufferfeld                                }
{Puffer[]            :Anzahl noch ausstehender Zeichen im Ringpuffer      }
{Size                :Groesse des Ringpuffers                             }
{COMPort[]           :Basisadresse COMX                                   }
{LCRoff              :Registeroffset Line Control Register                }
{MCRoff              :Registeroffset Modem Control Register               }
{IIRoff              :Registeroffset Interrupt Identification Register    }
{RTSMask             :Bitmaske fuer Request To Send                       }
{IntCont             :Interruptcontroller 1 oder 2                        }
{Port8259[]          :Basisadressen und EOI-Kontrollregister der 8259     }
{EOI8259             :Bitmaske fuer Non Specific Interrupt Ende 8259      }
{XOFF                :Zeichencode fuer SW-Handshake-Uebertragungsperre    }
{XON                 :Zeichencode fuer SW-Hsndshake-Freigabe              }
{RXHand[]            :Handshake Empfangsstatus der Schnittstelle          }
{TXHand[]            :Handshake Sendestatus der Schnittstelle             }
{LSRoff              :Registeroffset Line Status Register                 }
{RDRMask             :Receiver Data Ready Bit im Line Status Register     }
{Overrun[]           :Ueberlaufstatus des Empfangsringpuffers             }
{-------------------------------------------------------------------------}

(*$F+*)
PROCEDURE V24HSInt1;
(*$F-*)
Interrupt;

Label   Ende;           {Sprungpunkt zur Geschwindigkeitsoptimierung}

BEGIN
  Inline ($FA);         {Interruptroutine darf nicht unterbrochen werden}

  {Bei 8250/16450 1 Zeichen, bei 16550 alle Zeichen im RX-FIFO einlesen}
  While (Port[COMPort[1]+LSRoff] AND RDRMask)=RDRMask DO
    BEGIN
      {Zeichen auslesen und fuer Puffereintrag vormerken}
      V24Puffer[1,INTZEI[1]] := Port[COMPort[1]];

      {Bei XOFF Sendefreigabestatus=False und das Zeichen XOFF ausfiltern}  
      IF Soft[1] AND (V24Puffer[1,INTZEI[1]]=XOFF) THEN
        BEGIN
          TXHand[1]:=false;
          Goto Ende;
        END;

      {Bei XON Sendefreigabestatus=True und das Zeichen XON ausfiltern}
      IF Soft[1] AND (V24Puffer[1,INTZEI[1]]=XON) THEN
        BEGIN
          TXHand[1]:=true;
          Goto Ende;
        END;

      {Ist das vorgemerkte Zeichen bei Softwarehandshake kein XON/XOFF so}
      {wird es entgueltig durch Anpassung der Pufferzeiger in den Puffer }
      {eingetragen}
      {Nachlauf-Ringpuffer}
      IF INTZEI[1]>=SIZE THEN INTZEI[1]:=1 ELSE Inc(INTZEI[1]);
      Inc (Puffer[1]);
      {Bei Pufferueberlauf enthaelt Ringpuffer die letzten SIZE Zeichen}
      IF Puffer[1]>SIZE THEN
        BEGIN
          IF LESZEI[1]>=SIZE THEN LESZEI[1]:=1 ELSE INC (LESZEI[1]);
          Puffer[1]:=SIZE;
          Overrun[1]:=TRUE;
        END
      ELSE Overrun[1]:=False;

      {Handshake sperren bei drohendem Pufferueberlauf}
      IF Puffer[1] >= (SIZE-RESERVE) THEN
        BEGIN
          {Bei Hardwarehandshake RTS wegnehmen}
          IF Hard[1] THEN ClearRTS(1);
          {Bei Softwarehandshake XOFF ausgeben}
          IF Soft[1] THEN V24Write(1,Chr(XOFF));
          RXHand[1]:=false;
        END;

ENDE: {Sprungpunkt zur Geschwindigkeitsoptimierung}

    END;

  {Non spec. EOI fuer 2. 8259 bei IRQ 8-15}
  IF IntCont=2 THEN Port[Port8259[2]]:=EOI8259;

  {Non spec. EOI fuer 1. 8259 bei IRQ 1-7}
  Port[Port8259[1]]:=EOI8259;

  Inline ($FB);         {Interrupts wieder freigeben}
END;


{-------------------------------------------------------------------------}
{Procedure V24HSInt2                                                      }
{Diese Procedure ist eine Ringpuffer-Interrupt Routine mit automatischer  }
{Handshakesteuerung der Empfangsschnittstelle und Setzen des Sperrhand-   }
{shake bei drohendem Pufferueberlauf                                      }
{Bei Softwarehandshake Setzen des eigenen Sendestatus TXHand und Aus-     }
{filtern von XON und XOFF                                                 }
{Interne Konstanten:                                                      }
{Reserve             :Ausloeseabstand zum Pufferende fuer Handshakesperre }
{Globale Konstanten und Variable:                                         }
{INTZEI[]            :Schreibzeiger auf Ringpufferfeld                    }
{LESZEI[]            :Lesezeiger auf Ringpufferfeld                       }
{V24Puffer[INTZEI[]  :RS232-Ringpufferfeld                                }
{Puffer[]            :Anzahl noch ausstehender Zeichen im Ringpuffer      }
{Size                :Groesse des Ringpuffers                             }
{COMPort[]           :Basisadresse COMX                                   }
{LCRoff              :Registeroffset Line Control Register                }
{MCRoff              :Registeroffset Modem Control Register               }
{IIRoff              :Registeroffset Interrupt Identification Register    }
{RTSMask             :Bitmaske fuer Request To Send                       }
{IntCont             :Interruptcontroller 1 oder 2                        }
{Port8259[]          :Basisadressen und EOI-Kontrollregister der 8259     }
{EOI8259             :Bitmaske fuer Non Specific Interrupt Ende 8259      }
{XOFF                :Zeichencode fuer SW-Handshake-Uebertragungsperre    }
{XON                 :Zeichencode fuer SW-Hsndshake-Freigabe              }
{RXHand[]            :Handshake Empfangsstatus der Schnittstelle          }
{TXHand[]            :Handshake Sendestatus der Schnittstelle             }
{LSRoff              :Registeroffset Line Status Register                 }
{RDRMask             :Receiver Data Ready Bit im Line Status Register     }
{Overrun[]           :Ueberlaufstatus des Empfangsringpuffers             }
{-------------------------------------------------------------------------}

(*$F+*)
PROCEDURE V24HSInt2;
(*$F-*)
Interrupt;

Label   Ende;           {Sprungpunkt zur Geschwindigkeitsoptimierung}

BEGIN
  Inline ($FA);         {Interruptroutine darf nicht unterbrochen werden}

  {Bei 8250/16450 1 Zeichen, bei 16550 alle Zeichen im RX-FIFO einlesen}
  While (Port[COMPort[2]+LSRoff] AND RDRMask)=RDRMask DO
    BEGIN
      {Zeichen auslesen und fuer Puffereintrag vormerken}
      V24Puffer[2,INTZEI[2]] := Port[COMPort[2]];

      {Bei XOFF Sendefreigabestatus=False und das Zeichen XOFF ausfiltern}  
      IF Soft[2] AND (V24Puffer[2,INTZEI[2]]=XOFF) THEN
        BEGIN
          TXHand[2]:=false;
          Goto Ende;
        END;

      {Bei XON Sendefreigabestatus=True und das Zeichen XON ausfiltern}
      IF Soft[2] AND (V24Puffer[2,INTZEI[2]]=XON) THEN
        BEGIN
          TXHand[2]:=true;
          Goto Ende;
        END;

      {Ist das vorgemerkte Zeichen bei Softwarehandshake kein XON/XOFF so}
      {wird es entgueltig durch Anpassung der Pufferzeiger in den Puffer }
      {eingetragen}
      {Nachlauf-Ringpuffer}
      IF INTZEI[2]>=SIZE THEN INTZEI[2]:=1 ELSE Inc(INTZEI[2]);
      Inc (Puffer[2]);
      {Bei Pufferueberlauf enthaelt Ringpuffer die letzten SIZE Zeichen}
      IF Puffer[2]>SIZE THEN
        BEGIN
          IF LESZEI[2]>=SIZE THEN LESZEI[2]:=1 ELSE INC (LESZEI[2]);
          Puffer[2]:=SIZE;
          Overrun[2]:=TRUE;
        END
      ELSE Overrun[2]:=False;

      {Handshake sperren bei drohendem Pufferueberlauf}
      IF Puffer[2] >= (SIZE-RESERVE) THEN
        BEGIN
          {Bei Hardwarehandshake RTS wegnehmen}
          IF Hard[2] THEN ClearRTS(2);
          {Bei Softwarehandshake XOFF ausgeben}
          IF Soft[2] THEN V24Write(2,Chr(XOFF));
          RXHand[2]:=false;
        END;

ENDE: {Sprungpunkt zur Geschwindigkeitsoptimierung}

    END;

  {Non spec. EOI fuer 2. 8259 bei IRQ 8-15}
  IF IntCont=2 THEN Port[Port8259[2]]:=EOI8259;

  {Non spec. EOI fuer 1. 8259 bei IRQ 1-7}
  Port[Port8259[1]]:=EOI8259;

  Inline ($FB);         {Interrupts wieder freigeben}
END;


{-------------------------------------------------------------------------}
{Procedure V24HSInt3                                                      }
{Diese Procedure ist eine Ringpuffer-Interrupt Routine mit automatischer  }
{Handshakesteuerung der Empfangsschnittstelle und Setzen des Sperrhand-   }
{shake bei drohendem Pufferueberlauf                                      }
{Bei Softwarehandshake Setzen des eigenen Sendestatus TXHand und Aus-     }
{filtern von XON und XOFF                                                 }
{Interne Konstanten:                                                      }
{Reserve             :Ausloeseabstand zum Pufferende fuer Handshakesperre }
{Globale Konstanten und Variable:                                         }
{INTZEI[]            :Schreibzeiger auf Ringpufferfeld                    }
{LESZEI[]            :Lesezeiger auf Ringpufferfeld                       }
{V24Puffer[INTZEI[]] :RS232-Ringpufferfeld                                }
{Puffer[]            :Anzahl noch ausstehender Zeichen im Ringpuffer      }
{Size                :Groesse des Ringpuffers                             }
{COMPort[]           :Basisadresse COMX                                   }
{LCRoff              :Registeroffset Line Control Register                }
{MCRoff              :Registeroffset Modem Control Register               }
{IIRoff              :Registeroffset Interrupt Identification Register    }
{RTSMask             :Bitmaske fuer Request To Send                       }
{IntCont             :Interruptcontroller 1 oder 2                        }
{Port8259[]          :Basisadressen und EOI-Kontrollregister der 8259     }
{EOI8259             :Bitmaske fuer Non Specific Interrupt Ende 8259      }
{XOFF                :Zeichencode fuer SW-Handshake-Uebertragungsperre    }
{XON                 :Zeichencode fuer SW-Hsndshake-Freigabe              }
{RXHand[]            :Handshake Empfangsstatus der Schnittstelle          }
{TXHand[]            :Handshake Sendestatus der Schnittstelle             }
{LSRoff              :Registeroffset Line Status Register                 }
{RDRMask             :Receiver Data Ready Bit im Line Status Register     }
{Overrun[]           :Ueberlaufstatus des Empfangsringpuffers             }
{-------------------------------------------------------------------------}

(*$F+*)
PROCEDURE V24HSInt3;
(*$F-*)
Interrupt;

Label   Ende;           {Sprungpunkt zur Geschwindigkeitsoptimierung}

BEGIN
  Inline ($FA);         {Interruptroutine darf nicht unterbrochen werden}

  {Bei 8250/16450 1 Zeichen, bei 16550 alle Zeichen im RX-FIFO einlesen}
  While (Port[COMPort[3]+LSRoff] AND RDRMask)=RDRMask DO
    BEGIN
      {Zeichen auslesen und fuer Puffereintrag vormerken}
      V24Puffer[3,INTZEI[3]] := Port[COMPort[3]];

      {Bei XOFF Sendefreigabestatus=False und das Zeichen XOFF ausfiltern}  
      IF Soft[3] AND (V24Puffer[3,INTZEI[3]]=XOFF) THEN
        BEGIN
          TXHand[3]:=false;
          Goto Ende;
        END;

      {Bei XON Sendefreigabestatus=True und das Zeichen XON ausfiltern}
      IF Soft[3] AND (V24Puffer[3,INTZEI[3]]=XON) THEN
        BEGIN
          TXHand[3]:=true;
          Goto Ende;
        END;

      {Ist das vorgemerkte Zeichen bei Softwarehandshake kein XON/XOFF so}
      {wird es entgueltig durch Anpassung der Pufferzeiger in den Puffer }
      {eingetragen}
      {Nachlauf-Ringpuffer}
      IF INTZEI[3]>=SIZE THEN INTZEI[3]:=1 ELSE Inc(INTZEI[3]);
      Inc (Puffer[3]);
      {Bei Pufferueberlauf enthaelt Ringpuffer die letzten SIZE Zeichen}
      IF Puffer[3]>SIZE THEN
        BEGIN
          IF LESZEI[3]>=SIZE THEN LESZEI[3]:=1 ELSE INC (LESZEI[3]);
          Puffer[3]:=SIZE;
          Overrun[3]:=TRUE;
        END
      ELSE Overrun[3]:=False;

      {Handshake sperren bei drohendem Pufferueberlauf}
      IF Puffer[3] >= (SIZE-RESERVE) THEN
        BEGIN
          {Bei Hardwarehandshake RTS wegnehmen}
          IF Hard[3] THEN ClearRTS(3);
          {Bei Softwarehandshake XOFF ausgeben}
          IF Soft[3] THEN V24Write(3,Chr(XOFF));
          RXHand[3]:=false;
        END;

ENDE: {Sprungpunkt zur Geschwindigkeitsoptimierung}

    END;

  {Non spec. EOI fuer 2. 8259 bei IRQ 8-15}
  IF IntCont=2 THEN Port[Port8259[2]]:=EOI8259;

  {Non spec. EOI fuer 1. 8259 bei IRQ 1-7}
  Port[Port8259[1]]:=EOI8259;

  Inline ($FB);         {Interrupts wieder freigeben}
END;


{-------------------------------------------------------------------------}
{Procedure V24HSInt4                                                      }
{Diese Procedure ist eine Ringpuffer-Interrupt Routine mit automatischer  }
{Handshakesteuerung der Empfangsschnittstelle und Setzen des Sperrhand-   }
{shake bei drohendem Pufferueberlauf                                      }
{Bei Softwarehandshake Setzen des eigenen Sendestatus TXHand und Aus-     }
{filtern von XON und XOFF                                                 }
{Interne Konstanten:                                                      }
{Reserve             :Ausloeseabstand zum Pufferende fuer Handshakesperre }
{Globale Konstanten und Variable:                                         }
{INTZEI[]            :Schreibzeiger auf Ringpufferfeld                    }
{LESZEI[]            :Lesezeiger auf Ringpufferfeld                       }
{V24Puffer[INTZEI[]] :RS232-Ringpufferfeld                                }
{Puffer[]            :Anzahl noch ausstehender Zeichen im Ringpuffer      }
{Size                :Groesse des Ringpuffers                             }
{COMPort[]           :Basisadresse COMX                                   }
{LCRoff              :Registeroffset Line Control Register                }
{MCRoff              :Registeroffset Modem Control Register               }
{IIRoff              :Registeroffset Interrupt Identification Register    }
{RTSMask             :Bitmaske fuer Request To Send                       }
{IntCont             :Interruptcontroller 1 oder 2                        }
{Port8259[]          :Basisadressen und EOI-Kontrollregister der 8259     }
{EOI8259             :Bitmaske fuer Non Specific Interrupt Ende 8259      }
{XOFF                :Zeichencode fuer SW-Handshake-Uebertragungsperre    }
{XON                 :Zeichencode fuer SW-Hsndshake-Freigabe              }
{RXHand[]            :Handshake Empfangsstatus der Schnittstelle          }
{TXHand[]            :Handshake Sendestatus der Schnittstelle             }
{LSRoff              :Registeroffset Line Status Register                 }
{RDRMask             :Receiver Data Ready Bit im Line Status Register     }
{Overrun[]           :Ueberlaufstatus des Empfangsringpuffers             }
{-------------------------------------------------------------------------}

(*$F+*)
PROCEDURE V24HSInt4;
(*$F-*)
Interrupt;

Label   Ende;           {Sprungpunkt zur Geschwindigkeitsoptimierung}

BEGIN
  Inline ($FA);         {Interruptroutine darf nicht unterbrochen werden}

  {Bei 8250/16450 1 Zeichen, bei 16550 alle Zeichen im RX-FIFO einlesen}
  While (Port[COMPort[4]+LSRoff] AND RDRMask)=RDRMask DO
    BEGIN
      {Zeichen auslesen und fuer Puffereintrag vormerken}
      V24Puffer[4,INTZEI[4]] := Port[COMPort[4]];

      {Bei XOFF Sendefreigabestatus=False und das Zeichen XOFF ausfiltern}  
      IF Soft[4] AND (V24Puffer[4,INTZEI[4]]=XOFF) THEN
        BEGIN
          TXHand[4]:=false;
          Goto Ende;
        END;

      {Bei XON Sendefreigabestatus=True und das Zeichen XON ausfiltern}
      IF Soft[4] AND (V24Puffer[4,INTZEI[4]]=XON) THEN
        BEGIN
          TXHand[4]:=true;
          Goto Ende;
        END;

      {Ist das vorgemerkte Zeichen bei Softwarehandshake kein XON/XOFF so}
      {wird es entgueltig durch Anpassung der Pufferzeiger in den Puffer }
      {eingetragen}
      {Nachlauf-Ringpuffer}
      IF INTZEI[4]>=SIZE THEN INTZEI[4]:=1 ELSE Inc(INTZEI[4]);
      Inc (Puffer[4]);
      {Bei Pufferueberlauf enthaelt Ringpuffer die letzten SIZE Zeichen}
      IF Puffer[4]>SIZE THEN
        BEGIN
          IF LESZEI[4]>=SIZE THEN LESZEI[4]:=1 ELSE INC (LESZEI[4]);
          Puffer[4]:=SIZE;
          Overrun[4]:=TRUE;
        END
      ELSE Overrun[4]:=False;

      {Handshake sperren bei drohendem Pufferueberlauf}
      IF Puffer[4] >= (SIZE-RESERVE) THEN
        BEGIN
          {Bei Hardwarehandshake RTS wegnehmen}
          IF Hard[4] THEN ClearRTS(4);
          {Bei Softwarehandshake XOFF ausgeben}
          IF Soft[4] THEN V24Write(4,Chr(XOFF));
          RXHand[4]:=false;
        END;

ENDE: {Sprungpunkt zur Geschwindigkeitsoptimierung}

    END;

  {Non spec. EOI fuer 2. 8259 bei IRQ 8-15}
  IF IntCont=2 THEN Port[Port8259[2]]:=EOI8259;

  {Non spec. EOI fuer 1. 8259 bei IRQ 1-7}
  Port[Port8259[1]]:=EOI8259;

  Inline ($FB);         {Interrupts wieder freigeben}
END;


{-------------------------------------------------------------------------}
{Procedure V24OPEN                                                        }
{Diese Procedure installiert die interruptroutine und initialisiert die   }
{Statusvariablen fuer das Handshakeverfahren                              }
{Interfacevariablen                                                       }
{SER                 :Schnittstellennummer                                }
{BAUD                :Baudrate                                            }
{Bitzahl             :Wortlaenge                                          }
{Paritaet            :Paritaet                                            }
{Stopbit             :Stopbitzahl                                         }
{Handshake           :Handshake                                           }
{Globale Variablen und Konstanten:                                        }
{LODIVoff            :Registeroffset niederw. Baudratenteilerbyte         }
{HiDIVoff            :Registeroffset hoeherw. Baudratenteilerbyte         }
{ENDIVMask           :Bitmaske fuer Zugriff auf Baudratenteilerwort       }
{LCRSTOP[STOP]       :Feld fuer Stopbbitbitmaske 1Stopbit 2Stopbit        }
{keiPar              :Bitmasken fuer Paritaetsbiteinstellung              }
{ungPar                                                                   }
{gerPar                                                                   }
{DIVIS               :Baudratenteilerfaktor                               }
{LCR                 :Line Control register                               }
{LCRParit            :Paritaetsmaske                                      }
{Handshake           :Definition des Handshake "S"oftware,"Hardware,"K"ein}
{HARD[],SOFT[]       :Binaere Statusvariablen fuer aktiven Handshakemodus }
{COMPort[]           :Basisadresse COMX                                   }
{IERoff              :Registeroffset Interrupt Control Register COM-Port  }
{ICRoff              :Interruptmaskenoffset des 8259-Interruptcontoller   }
{IntCont             :Interruptcontroller 1 oder 2                        }
{Port8259[]          :Basisadressen und EOI-Kontrollregister der 8259     }
{V24PrioMask         :Maske zum Optimieren der RS232-IRQ-Prioritaet       }
{OrgIntMask[]        :Zwischenspeicher fuer System-Interruptmaske         }
{OrgIntVek[]         :Systeminterruptvektor der seriellen Karte           }
{OrgLODIV[]          :Alter Baudratenteilerfaktor vor Programmstart       }
{OrgHIDIV[]          :Alter Baudratenteilerfaktor vor Programmstart       }
{OrgLCR[]            :Altes Line Control Register vor Programmstart       }
{OrgMCR[]            :Altes MODEM-Control Register vor Programmstart      }
{OrgIER[]            :Altes Interrupt Enable Register vor Programmstart   }
{COMIRQ[]            :Feld der Interruptleitungen IRQs der ser. Karten    }
{IntMask[COMIRQ[]]   :Den IRQs zugeordneten Maskenbits im 8259            }
{COMInt[COMIRQ[]]    :Den IRQs im PC zugeordnete Interruptvektoren        }
{LESZEI[]            :Ringpufferfeldindex fuer Ausleseroutine V24READ     }
{INTZEI[]            :Ringpufferfeldindex fuer Interruptroutine           }
{Puffer[]            :Anzahl auszulesender Zeichen im Ringpuffer          }
{TXHand[]            :Handshakestatus Sendeschnittstelle                  }
{RXHand[]            :Handshakestatus Empfangsschnittstelle               }
{MCRoff              :Registeroffset Modem Control Register               }
{IRQMask             :Bitmaske Modem Control Register (OUT2=INT-Freigabe) }
{DTRMask             :Bitmaske Data Terminal Ready                        }
{RTSMask             :Bitmaske Request To SEND                            }
{FCRoff              :Registeroffset FIFO Control Register beim 16550     }
{FCRByte[]           :FIFO Control Register Konfigurationsbytes           }
{Overrun[]           :Ueberlaufstatus des Empfangsringpuffers             }
{V24Init[]           :Intitialisierungsstatus fuer COMx                   }
{-------------------------------------------------------------------------}

Procedure V24OPEN;

CONST   LCRSTOP         :ARRAY[1..2] of BYTE = (0,4);
        keiPar          =0;     {Bitmaske keine Paritaet}
        ungPar          =8;     {Bitmaske ungerade Paritaet}
        gerPar          =24;    {Bitmaske gerade Paritaet}

VAR     DIVIS           :INTEGER; {Baudratenteilerfaktor}
        LCR             :BYTE;  {Line Controlregister}
        LCRParit        :BYTE;  {Paritaetsbitmaske}

BEGIN
  {Pruefen ob bereits ein Schnittstellenhandler aktiviert wurde.}
  IF V24Init[SER] THEN EXIT;

  {Initialisierung der Pufferzeiger und Statusvariablen}
  LESZEI[SER] :=1;              {Pufferzeiger in Grundstellung}
  INTZEI[SER] :=1;
  Puffer[SER] :=0;
  TXHand[SER] :=TRUE;           {Statusvariablen einschalten}
  RXHand[SER] :=TRUE;
  Overrun[SER]:=False;          {Overrunstatus loeschen}

  {TFPCR-Software-Interface angewaehlt}
  IF SER=5 THEN
    BEGIN
      V24Init[SER]:=TFPCRInstall;   {Bei TFPCR nur Status setzen}
      SOFT[SER]  :=(Handshake='S'); {TFPCX Handshake ist ein Softwarehandshake}
      HARD[SER]  :=FALSE;           {Hardwarehandshake ausschalten}
      EXIT;
    END;

  {Interruptcontroller 1 (IRQ 1-7) oder 2 (IRQ 8-15) festlegen}
  IF COMIRQ[SER] <8 THEN IntCont:=1 ELSE IntCont:=2;

  {Handshakemodus festlegen: 'K'=kein HS 'S'=XON/XOFF 'H'=RTS/CTS/DSR}
  IF     (Handshake<>'K')      {Bei ungueltigem Parameter 'K' zuweisen}
     AND (Handshake<>'H')
     AND (Handshake<>'S') THEN Handshake:='K';
  Hard[SER]:= (Handshake='H');
  Soft[SER]:= (Handshake='S'); 

  {Alte UART-Register vor dem Neuinitialisieren retten}
  OrgLCR[SER]:=PORT[COMPort[SER]+LCRoff];     {altes LCR sichern}
  OrgMCR[SER]:=PORT[COMPort[SER]+MCRoff];     {altes MCR sichern}
  OrgIER[SER]:=PORT[COMPort[SER]+IERoff];     {altes IER sichern}
  PORT[COMPort[SER]+LCRoff]:=PORT[COMPort[SER]+LCRoff] OR ENDIVMASK;
  OrgLODIV[SER]:=PORT[COMPort[SER]+LODIVoff]; {alte Baudrate sichern}
  OrgHIDIV[SER]:=PORT[COMPort[SER]+HIDIVoff];
  PORT[COMPort[SER]+LCRoff]:=OrgLCR[SER];

  {Interruptcontroller scharfschalten und Interruptvektor installieren}
  Inline ($FA);                 {Interrupts bei INT-Definition sperren}
  CASE IntCont of
    1: BEGIN
         GetIntVec(COMInt[COMIRQ[SER]],OrgIntVek[SER,1]);
         OrgIntMask[SER,1]:=PORT[Port8259[1]+ICRoff];
         PORT[Port8259[1]+ICRoff]:=OrgIntMask[SER,1] AND (255-IntMask[COMIRQ[SER]]);
       END;
    2: BEGIN
         GetIntVec(COMInt[COMIRQ[SER]],OrgIntVek[SER,1]);
         GetIntVec(COMInt[COMIRQ[SER]],OrgIntVek[SER,2]);
         OrgIntMask[SER,1]:=PORT[Port8259[1]+ICRoff];
         OrgIntMask[SER,2]:=PORT[Port8259[2]+ICRoff];
         PORT[Port8259[2]+ICRoff]:=OrgIntMask[SER,2] AND (255-IntMask[COMIRQ[SER]]);
         PORT[Port8259[1]+ICRoff]:=OrgIntMask[SER,1] AND (255-4);
       END;
    END;        
  CASE SER of
    1: IF (Handshake='K') THEN SetIntVec(COMInt[COMIRQ[SER]],@V24RingInt1)
       ELSE SetIntVec(COMInt[COMIRQ[SER]],@V24HSInt1);
    2: IF (Handshake='K') THEN SetIntVec(COMInt[COMIRQ[SER]],@V24RingInt2)
       ELSE SetIntVec(COMInt[COMIRQ[SER]],@V24HSInt2);
    3: IF (Handshake='K') THEN SetIntVec(COMInt[COMIRQ[SER]],@V24RingInt3)
       ELSE SetIntVec(COMInt[COMIRQ[SER]],@V24HSInt3);
    4: IF (Handshake='K') THEN SetIntVec(COMInt[COMIRQ[SER]],@V24RingInt4)
       ELSE SetIntVec(COMInt[COMIRQ[SER]],@V24HSInt4);
    END;
  PORT[Port8259[1]]:=V24PrioMask;       {V24 IRQ-Prioritaet setzen}
  Inline ($FB);                         {Interrupts freigeben}

  {Seriellen Port initialisieren und IRQ freigeben}

  {Baudrate berechnen}
  DIVIS:=Round(115200/Baud);
  {LCR-Register aufbauen}
  IF      Paritaet='U' THEN LCRParit:=ungPar
  ELSE IF Paritaet='G' THEN LCRParit:=gerPar
                       ELSE LCRParit:=keiPar;
  LCR:=0;
  LCR:=LCR + (Bitzahl-5);               {Bitzahl 5-8}
  LCR:=LCR + LCRSTOP[Stopbit];          {Stopbits 1-2}
  LCR:=LCR + LCRPARIT;                  {Paritaet hier 'k'eine 'u'ng. 'g'er.}

  {UART initialisieren}

  {UART-Status bereinigen}
  SIOClear(SER);                        {SIO-Status bereinigen}

  {Baudrate einstellen}
  PORT[COMPort[SER]+LCRoff]:=PORT[COMPort[SER]+LCRoff] OR ENDIVMASK;
  PORT[COMPort[SER]+LODIVoff]:=LO(DIVIS);
  PORT[COMPOrt[SER]+HIDIVoff]:=HI(DIVIS);

  {Uebertragungsparameter einstellen}
  PORT[COMPort[SER]+LCRoff]:=LCR;

  {FIFO-Controlregister eines NSC 16550A initialisieren sofern installiert}
  {Beim 8250/16450 geht diese Initialisierung ins Leere und bewirkt nichts}
  Port[COMPort[SER]+FCRoff]:=FCRByte[SER];

  {Den UART auf Interruptsendung bei Zeichenempfang programmieren}
  PORT[COMPort[SER]+IERoff]:=01;

  {Hardwarehandshakeleitungen und IRQ-Tristatetreiber aktivieren}
  PORT[COMPort[SER]+MCRoff]:=IRQMask+DTRMask+RTSMask;
  If Soft[SER] THEN V24Write(SER,Chr(XON)); {Freigabe an Endgeraet melden}
  V24Init[SER]:=TRUE;
END;


{-------------------------------------------------------------------------}
{Procedure V24Close                                                       }
{Diese Procedure entfernt die Interruptroutine und reinitialisiert die    }
{alten Interrupt- und SIO-Register und loescht die Statusvariablen        }
{Globale Variablen und Konstanten:                                        }
{HARD[],SOFT[]       :Binaere Statusvariablen fuer aktiven Handshakemodus }
{SER                 :Schnittstellennummer COM"X"                         }
{COMPort[SER]        :Basisadresse COMX                                   }
{IERoff              :Registeroffset Interrupt Control Register COM-Port  }
{Port8259[]          :Basisadressen und EOI-Kontrollregister der 8259     }
{ICRoff              :Interruptmaskenoffset des 8259-Interruptcontoller   }
{StdPrioMask         :Standard IRQ-Prioritaet im PC-System                }
{OrgIntMask[]        :Zwischenspeicher fuer System-Interruptmaske         }
{OrgIntVek[]         :Systeminterruptvektor der seriellen Karte           }
{OrgLODIV[]          :Alter Baudratenteilerfaktor vor Programmstart       }
{OrgHIDIV[]          :Alter Baudratenteilerfaktor vor Programmstart       }
{OrgLCR[]            :Altes Line Control Register vor Programmstart       }
{OrgMCR[]            :Altes MODEM-Control Register vor Programmstart      }
{OrgIER[]            :Altes Interrupt Enable Register vor Programmstart   }
{IntMask[COMIRQ[]]   :Den IRQs zugeordneten Maskenbits im 8259            }
{COMInt[COMIRQ[]]    :Den IRQs im PC zugeordnete Interruptvektoren        }
{RXHand[]            :Handshakestatus Empfangsschnittstelle               }
{V24Init[]           :Intitialisierungsstatus fuer COMx                   }
{-------------------------------------------------------------------------}

Procedure V24CLOSE;

BEGIN
  FOR SER:=1 TO 5 DO IF V24Init[SER] THEN     {Alles schliessen was offen war}
    BEGIN

      {TFPCR-Software-Interface angewaehlt}
      IF SER=5 THEN
        BEGIN
          V24Init[SER]:=False;                {Bei TFPCR nur Status}
          EXIT;                               {abschalten und fertig}
        END;

      PORT[COMPort[SER]+IERoff]:=0;           {serielles Port-IER sperren}
      PORT[COMPort[SER]+MCRoff]:=0;           {IRQ-Leitung in Tristate}

      SIOClear(SER);                          {SIO-Status bereinigen}
      Inline($FA);                            {Interrupts sperren}
 
      CASE IntCont OF
           {IRQ 1-7 mit einem 8259}
        1: BEGIN
             PORT[Port8259[1]+ICRoff]:=OrgIntMask[SER,1];     
             SetIntVec(COMInt[COMIRQ[SER]],OrgIntVek[SER,1]);
             PORT[Port8259[1]]:=StdPrioMask;
           END;
           {IRQ 8-15 (2) mit zwei 8259 auf AT}
        2: BEGIN
             PORT[Port8259[2]+ICRoff]:=OrgIntMask[SER,2];     
             PORT[Port8259[1]+ICRoff]:=OrgIntMask[SER,1];     
             SetIntVec(COMInt[COMIRQ[SER]],OrgIntVek[SER,2]);
             SetIntVec(COMInt[2],OrgIntVek[SER,2]);
             PORT[Port8259[1]]:=StdPrioMask;
           END;
      END;

      Hard[SER]:=false;
      Soft[SER]:=false;

      {Alte SIO-Einstellung vor dem Programmstart wieder einstellen}
      PORT[COMPort[SER]+LCRoff]:=PORT[COMPort[SER]+LCRoff] OR ENDIVMASK;
      PORT[COMPort[SER]+LODIVoff]:=OrgLODIV[SER]; {alte Baudrate restaurieren}
      PORT[COMPort[SER]+HIDIVoff]:=OrgHIDIV[SER];
      PORT[COMPort[SER]+LCRoff]:=OrgLCR[SER];     {LCR restaurieren}
      PORT[COMPort[SER]+IERoff]:=OrgIER[SER];     {IER restaurieren}

      Inline($FB);                                {Interrupts freigeben}

      SIOClear(SER);                              {SIO-Status bereinigen}
      PORT[COMPort[SER]+MCRoff]:=OrgMCR[SER];     {MCR restaurieren}
      V24Init[SER]:=false;
    END;
END;


{**************************************************************************}
{ Position des spaeteren Anwender-Hauptprogramms                           }
{**************************************************************************}

BEGIN
END.
